#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

use HTML::Template;
use ActiveState::Config qw(:path);
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Widget;
use PureMessage::Enduser::Config qw(%config);
use PureMessage::Manager::Lang;
use PureMessage::Manager::Response::Quarantine::Enduser;
use strict;

my %vars;

###############################################################################
# Specify the paths of the config items we're updating.
###############################################################################
my %PATHS = (
    'AUTHENTICATORS'    => ['Authenticator#multi.config.authenticators', '#string'],
    );

###############################################################################
# Create response object.
###############################################################################
my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Quarantine::Enduser',
                                'stderr_to_stdout' => 1 );
$r->title(text('enduserf_multiauth_title'));
$r->{debug} = 0;
$r->addRestartDlg(type=>'httpd');

###############################################################################
# No custom help pages available for this add-on package yet.
###############################################################################
#$r->add_help("About This Page", "mgr/mgr_locserv_enduser_multiauth");
#$r->add_help("User Guide: <br>&nbsp;&nbsp; Multi Authenticator", "services", "enduser_multiauth");

###############################################################################
# Get form data, and determine action to perform.
###############################################################################
my $action = $in{'action'};
$action = 'save' if ($in{'save'});

$in{'load'} = 1 if (!$in{'revisit'});

###############################################################################
# If we're supposed to be saving the values, go try to save things to disk.
###############################################################################
if ($action =~ /save/) {
    if (!hasValidateErrors()) {
        commitUpdatedDataToDisk();
        # allow PM::Service::Config to not exist, like
        # for instance, if they have an older PM-Utils
        # without it.
        eval {
            require PureMessage::Service::Config;
            PureMessage::Service::Config::stale('httpd');
        };
    }
}
putEditPage();

###############################################################################
###############################################################################
sub putEditPage() {
    ###########################################################################
    # Add header to top of page.
    ###########################################################################
    $vars{HEADER} = $r->ThemeSectionHeader( title=>text('enduserf_multiauth_header'))->output();

    ###########################################################################
    # Template variables; load or preserve from form post.
    ###########################################################################
    if ($in{'load'}) {
        # Load
        foreach my $fname (keys %PATHS) {
            my $node = path_node( $PureMessage::Enduser::Config::object->root, $PATHS{$fname}->[0] );
            if ($fname eq 'AUTHENTICATORS') {
                # expecting an array
                $vars{$fname} = $node ? join(' ',$node->values()) : '';
            } else {
                $vars{$fname} = $node ? $node->value_formatted() : '';
            }
        }
    } else {
        # Preserve
        foreach my $fname (keys %PATHS) {
            $vars{$fname} = $in{$fname};
        }
    }

    ###########################################################################
    # Populate the list of "known authenticators".
    ###########################################################################
    my @known;
    foreach my $authname (sort keys %{$config{'Authenticator'}}) {
        my $authinfo = $config{'Authenticator'}{$authname};
        # Skip "Multi" authenticators.  No real reason other than trying to
        # make it less obvious to the user that you can multi a multi (which
        # although possible isn't really recommended).
        next if ($authinfo->{'module'} =~ /::Multi$/);
        # Remember this authenticator
        push( @known, { NAME => $authname, DESC => $authinfo->{'description'} } );
    }
    $vars{KNOWN_AUTHENTICATORS} = \@known;

    ###########################################################################
    # Create the toolbar, and add it to the footer on the page.
    ###########################################################################
    my $tb = $r->ToolBar(
                        buttons => [[ { type=>'image', text=>'Save', name=>'save', icon=>'save' },
                                      { type=>'image', text=>'Cancel', href=>'../index.cgi', icon=>'cancel' },
                                   ]],
                        table => [[1,3],['50%','50%']],
                );
    $vars{FOOTER} = $r->ThemeSectionFooter(toolbar=>$tb)->output();

    ###########################################################################
    # Add our template to the request, and send it to the browser.
    ###########################################################################
    $r->add_template( 'file'=>'tmpl/multiauth.html', 'data'=>\%vars );
    $r->send();
}

###############################################################################
###############################################################################
sub commitUpdatedDataToDisk {
    my %values;

    eval {
        my $conf = PureMessage::Enduser::Config::edit();
        foreach my $k (keys %PATHS) {
            if ($k eq 'AUTHENTICATORS') {
                # expecting an array
                my @v = split( /\s+/, $in{$k} );
                path_values( $conf->root, $PATHS{$k}->[0], @v );
            } else {
                path_value( $conf->root, $PATHS{$k}->[0], $in{$k} );
            }
        }
        $conf->save();
        PureMessage::Enduser::Config::refresh();
    };

    if ($@) {
        # config died, deal with it
        $r->raise_error('error_index_saving_conf',$@);
        $r->error_dlg();
        $r->send();
        exit;
    }

    $r->addInfoBox( msg => text('message_update_ok') );
    $r->log('Update Enduser Multi Authenticator', '', 'enduser', \%in );
}

###############################################################################
###############################################################################
sub hasValidateErrors {
    # Validate based on rule-set.
    foreach my $k (keys %PATHS) {
        $REQUEST->typedecl($k, $PATHS{$k}->[1], $PATHS{$k}->[2] );
    }
    my @v_errors = $REQUEST->validate( \%in );

    # Validate the authenticators against the list of known authenticators.
    my @auth  = split( /\s+/, $in{'AUTHENTICATORS'} );
    foreach (@auth) {
        if (!exists $config{'Authenticator'}{$_}) {
            push( @v_errors, "Unknown authenticator '$_'" );
        }
    }

    # Raise any errors.
    if (@v_errors) {
        $r->raise_error( join("<br>", @v_errors) );
    }
    return $r->error() ? 1 : 0;
}
